@@ -37,7 +37,7 @@ from . import settings
 TUTORIAL = '(tutorial and (pause + tutorial) or [])'
 MNEMONIC = '(mnemonic and (pause + mnemonic + lineBreak) or [])'
 
-BRAILLE_TEXT = '[Text(obj, asString(labelOrName + placeholderText), asString(eol), startOffset, endOffset)]\
+BRAILLE_TEXT = '[Text(obj, asString(labelOrName or placeholderText), asString(eol), startOffset, endOffset)]\
                 + (required and [Region(" " + asString(required))])\
                 + (readOnly and [Region(" " + asString(readOnly))])'
 
@@ -107,7 +107,7 @@ formatting = {
             'detailedWhereAmI' : '[]'
             },
         pyatspi.ROLE_ALERT: {
-            'unfocused': 'labelOrName + unrelatedLabels'
+            'unfocused': 'expandedEOCs or (labelOrName + unrelatedLabels)'
             },
         pyatspi.ROLE_ANIMATION: {
             'unfocused': 'labelAndName'
@@ -145,22 +145,18 @@ formatting = {
             },
         pyatspi.ROLE_DIALOG: {
             'focused': 'labelOrName',
-            'unfocused': 'labelOrName + unrelatedLabels'
+            'unfocused': 'expandedEOCs or (labelOrName + unrelatedLabels)'
             },
         pyatspi.ROLE_DOCUMENT_FRAME: {
             'unfocused': 'label + readOnly + textRole + textContent + anyTextSelection + ' + MNEMONIC,
             'basicWhereAmI': 'label + readOnly + textRole + textContent + anyTextSelection + ' + MNEMONIC,
             'detailedWhereAmI': 'label + readOnly + textRole + textContentWithAttributes + anyTextSelection + ' + MNEMONIC + ' + ' + TUTORIAL
             },
-        pyatspi.ROLE_EMBEDDED: {
-            'focused': 'embedded',
-            'unfocused': 'embedded'
-            },
         pyatspi.ROLE_ENTRY: {
-            'focused': 'labelOrName + placeholderText + readOnly + textRole + currentLineText + allTextSelection',
-            'unfocused': 'labelOrName + placeholderText + readOnly + textRole + currentLineText + allTextSelection + ' + MNEMONIC,
-            'basicWhereAmI': 'labelOrName + placeholderText + readOnly + textRole + textContent + anyTextSelection + ' + MNEMONIC,
-            'detailedWhereAmI': 'labelOrName + placeholderText + readOnly + textRole + textContentWithAttributes + anyTextSelection + ' + MNEMONIC + ' + ' + TUTORIAL
+            'focused': 'labelOrName + readOnly + textRole + (currentLineText or placeholderText) + allTextSelection',
+            'unfocused': 'labelOrName + readOnly + textRole + (currentLineText or placeholderText) + allTextSelection + ' + MNEMONIC,
+            'basicWhereAmI': 'labelOrName + readOnly + textRole + (textContent or placeholderText) + anyTextSelection + ' + MNEMONIC,
+            'detailedWhereAmI': 'labelOrName + readOnly + textRole + (textContentWithAttributes or placeholderText) + anyTextSelection + ' + MNEMONIC + ' + ' + TUTORIAL
             },
         pyatspi.ROLE_FORM: {
             'focused': 'labelOrName + readOnly + textRole + currentLineText + allTextSelection',
@@ -170,6 +166,9 @@ formatting = {
             'focused': 'labelOrName + roleName',
             'unfocused': 'labelOrName + allTextSelection + roleName + unfocusedDialogCount + availability'
             },
+        pyatspi.ROLE_HEADER: {
+            'unfocused': '(displayedText or name) + roleName',
+        },
         pyatspi.ROLE_HEADING: {
             'unfocused': 'displayedText + roleName + expandableState + availability + ' + MNEMONIC,
             'basicWhereAmI': 'label + readOnly + textRole + textContent + anyTextSelection + ' + MNEMONIC,
@@ -185,7 +184,9 @@ formatting = {
             'unfocused': 'labelAndName + unrelatedLabels'
             },
         pyatspi.ROLE_LABEL: {
-            'basicWhereAmI': 'labelOrName + allTextSelection + roleName'
+            'focused': 'labelAndName + allTextSelection + roleName',
+            'unfocused': 'labelAndName + allTextSelection + roleName',
+            'basicWhereAmI': 'labelAndName + allTextSelection + roleName'
             },
         pyatspi.ROLE_LAYERED_PANE: {
             'focused': 'labelAndName + allTextSelection + roleName + availability + noShowingChildren',
@@ -218,7 +219,7 @@ formatting = {
         pyatspi.ROLE_MENU_ITEM: {
             'focused': 'expandableState',
             'unfocused': 'labelOrName + menuItemCheckedState + expandableState + availability + ' + MNEMONIC + ' + accelerator + positionInList',
-            'basicWhereAmI': 'ancestors + labelOrName + accelerator + positionInList + ' + MNEMONIC
+            'basicWhereAmI': 'ancestors + parentRoleName + labelOrName + accelerator + positionInList + ' + MNEMONIC
             },
         pyatspi.ROLE_NOTIFICATION: {
             'unfocused': 'roleName + unrelatedLabels'
@@ -338,15 +339,15 @@ formatting = {
             'detailedWhereAmI': 'label + readOnly + textRole + textContentWithAttributes + anyTextSelection + ' + MNEMONIC + ' + ' + TUTORIAL
             },
         pyatspi.ROLE_TEXT: {
-            'focused': 'labelOrName + placeholderText + readOnly + textRole + textIndentation + currentLineText + allTextSelection',
-            'unfocused': 'labelOrName + placeholderText + readOnly + textRole + textIndentation + currentLineText + allTextSelection + ' + MNEMONIC,
-            'basicWhereAmI': 'label + placeholderText + readOnly + textRole + textContent + anyTextSelection + ' + MNEMONIC,
-            'detailedWhereAmI': 'label + placeholderText + readOnly + textRole + textContentWithAttributes + anyTextSelection + ' + MNEMONIC + ' + ' + TUTORIAL
+            'focused': 'labelOrName + readOnly + textRole + textIndentation + (currentLineText or placeholderText) + allTextSelection',
+            'unfocused': 'labelOrName + readOnly + textRole + textIndentation + (currentLineText or placeholderText) + allTextSelection + ' + MNEMONIC,
+            'basicWhereAmI': 'labelOrName + readOnly + textRole + (textContent or placeholderText) + anyTextSelection + ' + MNEMONIC,
+            'detailedWhereAmI': 'labelOrName + readOnly + textRole + (textContentWithAttributes or placeholderText) + anyTextSelection + ' + MNEMONIC + ' + ' + TUTORIAL
             },
         pyatspi.ROLE_TOGGLE_BUTTON: {
             'focused': 'expandableState or toggleState',
-            'unfocused': 'labelOrName + roleName + (expandableState or toggleState) + availability + ' + MNEMONIC + ' + accelerator',
-            'basicWhereAmI': 'labelOrName + roleName + (expandableState or toggleState)'
+            'unfocused': 'labelAndName + roleName + (expandableState or toggleState) + availability + ' + MNEMONIC + ' + accelerator',
+            'basicWhereAmI': 'labelAndName + roleName + (expandableState or toggleState)'
             },
         pyatspi.ROLE_TOOL_TIP: {
             'unfocused': 'labelAndName',
@@ -397,7 +398,10 @@ formatting = {
             'unfocused': '[Component(obj,\
                                      asString(label + displayedText + value + roleName + required))]',
             },
-        #pyatspi.ROLE_ALERT: 'default'
+        pyatspi.ROLE_ALERT: {
+            'unfocused': '((substring and ' + BRAILLE_TEXT + ')\
+                          or ([Component(obj, asString(labelAndName + roleName))]))'
+            },
         pyatspi.ROLE_ANIMATION: {
             'unfocused': '[Component(obj,\
                                      asString(label + displayedText + roleName + (description and space(": ") + description)))]',
@@ -423,9 +427,9 @@ formatting = {
             # We could then use the cursorOffset field to indicate where the
             # combobox starts.]]]
             #
-            'unfocused': '((comboBoxTextObj and [Text(comboBoxTextObj[0], asString(label), asString(eol))])\
-                           or [Component(obj, asString(label + name), label and (len(asString(label)) + 1) or 0)])\
-                          + [Region(" " + asString(roleName))]'
+            'unfocused': '((comboBoxTextObj and ([Text(comboBoxTextObj[0], asString(label), asString(eol))] \
+                                               + [Region(" " + asString(roleName))])) \
+                           or [Component(obj, asString(label + name + roleName), label and (len(asString(label)) + 1) or 0)])'
             },
         #pyatspi.ROLE_DESKTOP_ICON: 'default'
         pyatspi.ROLE_DIAL: {
@@ -442,13 +446,13 @@ formatting = {
                           + (readOnly and [Region(" " + asString(readOnly))])',
             'unfocused': BRAILLE_TEXT
             },
-        pyatspi.ROLE_EMBEDDED: {
-            'unfocused': '[Component(obj,\
-                                     asString(label + displayedText) or asString(applicationName))]'
-            },
         pyatspi.ROLE_ENTRY: {
             'unfocused': BRAILLE_TEXT
             },
+        pyatspi.ROLE_FORM: {
+            'unfocused': '((substring and ' + BRAILLE_TEXT + ')\
+                          or ([Component(obj, asString(labelAndName + roleName))]))'
+            },
         pyatspi.ROLE_FRAME: {
             'focused':   '[Component(obj,\
                                      asString(((label + displayedText) or name) + value + roleName + alertAndDialogCount))]',
@@ -456,7 +460,8 @@ formatting = {
                                      asString(((label + displayedText) or name) + value + roleName + alertAndDialogCount))]'
             },
         pyatspi.ROLE_HEADING: {
-            'unfocused': '[Text(obj)] + [Region(" " + asString(roleName))]'
+            'unfocused': '[Text(obj, asString(placeholderText), "", startOffset, endOffset)]\
+                          + [Region(" " + asString(roleName))]'
             },
         #pyatspi.ROLE_HTML_CONTAINER: 'default'
         pyatspi.ROLE_ICON: {
@@ -475,7 +480,8 @@ formatting = {
         pyatspi.ROLE_LINK: {
             'unfocused': '[Link(obj, asString(currentLineText)\
                                      or asString(displayedText)\
-                                     or asString(name))]',
+                                     or asString(name))] \
+                        + (roleName and [Region(" " + asString(roleName))])',
         },
         pyatspi.ROLE_LIST: {
             'unfocused': '[Component(obj,\
@@ -510,6 +516,9 @@ formatting = {
                                      asString(labelOrName + expandableState + availability) + asString(accelerator),\
                                      indicator=asString(menuItemCheckedState))]'
             },
+        pyatspi.ROLE_PAGE: {
+            'unfocused': BRAILLE_TEXT
+            },
         #pyatspi.ROLE_OPTION_PANE: 'default'
         pyatspi.ROLE_PAGE_TAB: {
             'focused':   '[Component(obj,\
@@ -687,13 +696,13 @@ if settings.useExperimentalSpeechProsody:
     formatting['speech'][pyatspi.ROLE_TERMINAL]['detailedWhereAmI'] = \
         'label + readOnly + pause + textRole + pause + textContentWithAttributes + anyTextSelection + ' + MNEMONIC + ' + ' + TUTORIAL
     formatting['speech'][pyatspi.ROLE_TEXT]['focused'] = \
-        'labelOrName + placeholderText + readOnly + textRole + pause + textIndentation + currentLineText + allTextSelection'
+        'labelOrName + readOnly + textRole + pause + textIndentation + (currentLineText or placeholderText) + allTextSelection'
     formatting['speech'][pyatspi.ROLE_TEXT]['unfocused'] = \
-        'labelOrName + placeholderText + readOnly + textRole + pause + textIndentation + currentLineText + allTextSelection + ' + MNEMONIC
+        'labelOrName + readOnly + textRole + pause + textIndentation + (currentLineText or placeholderText) + allTextSelection + ' + MNEMONIC
     formatting['speech'][pyatspi.ROLE_TEXT]['basicWhereAmI'] = \
-        'label + placeholderText + readOnly + textRole + pause + textContent + anyTextSelection + pause + ' + MNEMONIC
+        'labelOrName + readOnly + textRole + pause + (textContent or placeholderText) + anyTextSelection + pause + ' + MNEMONIC
     formatting['speech'][pyatspi.ROLE_TEXT]['detailedWhereAmI'] = \
-        'label + placeholderText + readOnly + textRole + pause + textContentWithAttributes + anyTextSelection + pause + ' + MNEMONIC + ' + ' + TUTORIAL
+        'labelOrName + readOnly + textRole + pause + (textContentWithAttributes or placeholderText) + anyTextSelection + pause + ' + MNEMONIC + ' + ' + TUTORIAL
 
 class Formatting(dict):
 
