@@ -34,9 +34,9 @@ import pyatspi
 from . import braille
 from . import debug
 from . import messages
+from . import object_properties
 from . import settings
 
-from .orca_i18n import _         # for gettext support
 import collections
 
 def _formatExceptionInfo(maxTBlevel=5):
@@ -301,6 +301,19 @@ class Generator:
         """
         result = []
         name = self._script.utilities.displayedText(obj)
+        if obj.getRole() == pyatspi.ROLE_COMBO_BOX:
+            children = self._script.utilities.selectedChildren(obj)
+            if not children and obj.childCount:
+                children = self._script.utilities.selectedChildren(obj[0])
+            children = children or [child for child in obj]
+            names = map(self._script.utilities.displayedText, children)
+            names = list(filter(lambda x: x, names))
+            if len(names) == 1:
+                name = names[0].strip()
+            elif len(children) == 1 and children[0].name:
+                name = children[0].name.strip()
+            elif not names and obj.name:
+                name = obj.name
         if name:
             result.append(name)
         else:
@@ -343,7 +356,7 @@ class Generator:
         result.extend(label)
         if not len(label):
             result.extend(name)
-        elif len(name) and name[0] != label[0]:
+        elif len(name) and name[0].strip() != label[0].strip():
             result.extend(name)
         return result
 
@@ -410,6 +423,26 @@ class Generator:
     #                                                                   #
     #####################################################################
 
+    def _generateClickable(self, obj, **args):
+        if not args.get('mode', None):
+            args['mode'] = self._mode
+        args['stringType'] = 'clickable'
+
+        if self._script.utilities.isClickableElement(obj):
+            return [self._script.formatting.getString(**args)]
+
+        return []
+
+    def _generateHasLongDesc(self, obj, **args):
+        if not args.get('mode', None):
+            args['mode'] = self._mode
+        args['stringType'] = 'haslongdesc'
+
+        if self._script.utilities.hasLongDesc(obj):
+            return [self._script.formatting.getString(**args)]
+
+        return []
+
     def _generateAvailability(self, obj, **args):
         """Returns an array of strings for use by speech and braille that
         represent the grayed/sensitivity/availability state of the
@@ -450,8 +483,7 @@ class Generator:
         if not args.get('mode', None):
             args['mode'] = self._mode
         args['stringType'] = 'readonly'
-        if settings.presentReadOnlyText \
-           and self._script.utilities.isReadOnlyTextArea(obj):
+        if self._script.utilities.isReadOnlyTextArea(obj):
             result.append(self._script.formatting.getString(**args))
         return result
 
@@ -462,21 +494,11 @@ class Generator:
         returned if this is not a checkable cell.
         """
         result = []
-        try:
-            action = obj.queryAction()
-        except NotImplementedError:
-            action = None
-        if action:
-            for i in range(0, action.nActions):
-                # Translators: this is the action name for
-                # the 'toggle' action. It must be the same
-                # string used in the *.po file for gail.
-                #
-                if action.getName(i) in ["toggle", _("toggle")]:
-                    oldRole = self._overrideRole(pyatspi.ROLE_CHECK_BOX,
-                                            args)
-                    result.extend(self.generate(obj, **args))
-                    self._restoreRole(oldRole, args)
+        if self._script.utilities.hasMeaningfulToggleAction(obj):
+            oldRole = self._overrideRole(pyatspi.ROLE_CHECK_BOX, args)
+            result.extend(self.generate(obj, **args))
+            self._restoreRole(oldRole, args)
+
         return result
 
     def _generateCheckedState(self, obj, **args):
@@ -564,8 +586,6 @@ class Generator:
         args['stringType'] = 'checkbox'
         indicators = self._script.formatting.getString(**args)
         if obj.getState().contains(pyatspi.STATE_CHECKED):
-            # Translators: this represents the state of a checked menu item.
-            #
             result.append(indicators[1])
         return result
 
@@ -600,56 +620,23 @@ class Generator:
         it exists.  Otherwise, an empty array is returned.
         """
         result = []
-
-        # Do not return yourself as a header.
-        #
-        role = args.get('role', obj.getRole())
-        if role in [pyatspi.ROLE_ROW_HEADER,
-                    pyatspi.ROLE_TABLE_ROW_HEADER]:
+        header = self._script.utilities.rowHeaderForCell(obj)
+        if not header:
             return result
 
-        if not args.get('mode', None):
-            args['mode'] = self._mode
-        try:
-            table = obj.parent.queryTable()
-        except:
-            pass
-        else:
-            index = self._script.utilities.cellIndex(obj)
-            try:
-                rowIndex = table.getRowAtIndex(index)
-            except:
-                rowIndex = -1
-            if rowIndex >= 0:
-                # Get the header information.  In Java Swing, the
-                # information is not exposed via the description
-                # but is instead a header object, so we fall back
-                # to that if it exists.
-                #
-                # [[[TODO: WDW - the more correct thing to do, I
-                # think, is to look at the row header object.
-                # We've been looking at the description for so
-                # long, though, that we'll give the description
-                # preference for now.]]]
-                #
-                desc = table.getRowDescription(rowIndex)
-                if not desc:
-                    header = table.getRowHeader(rowIndex)
-                    if header:
-                        desc = self._script.utilities.displayedText(header)
-                if desc and len(desc):
-                    text = desc
-                    if args['mode'] == 'speech':
-                        if settings.speechVerbosityLevel \
-                           == settings.VERBOSITY_LEVEL_VERBOSE \
-                           and not args.get('formatType', None) \
-                                   in ['basicWhereAmI', 'detailedWhereAmI']:
-                            text = desc + " " + self.getLocalizedRoleName(
-                                obj, pyatspi.ROLE_ROW_HEADER)
-                    elif args['mode'] == 'braille':
-                        text = desc + " " + self.getLocalizedRoleName(
-                            obj, pyatspi.ROLE_ROW_HEADER)
-                    result.append(text)
+        text = self._script.utilities.displayedText(header)
+        if not text:
+            return result
+
+        roleString =  self.getLocalizedRoleName(obj, pyatspi.ROLE_ROW_HEADER)
+        if args.get('mode') == 'speech':
+            if settings.speechVerbosityLevel == settings.VERBOSITY_LEVEL_VERBOSE \
+               and not args.get('formatType') in ['basicWhereAmI', 'detailedWhereAmI']:
+                text = "%s %s" % (text, roleString)
+        elif args.get('mode') == 'braille':
+            text = "%s %s" % (text, roleString)
+
+        result.append(text)
         return result
 
     def _generateColumnHeader(self, obj, **args):
@@ -659,54 +646,23 @@ class Generator:
         is returned.
         """
         result = []
-
-        # Do not return yourself as a header.
-        #
-        try:
-            role = args.get('role', obj.getRole())
-        except:
-            role = None
-        if role in [pyatspi.ROLE_COLUMN_HEADER,
-                    pyatspi.ROLE_TABLE_COLUMN_HEADER]:
+        header = self._script.utilities.columnHeaderForCell(obj)
+        if not header:
             return result
 
-        try:
-            table = obj.parent.queryTable()
-        except:
-            pass
-        else:
-            index = self._script.utilities.cellIndex(obj)
-            columnIndex = table.getColumnAtIndex(index)
-            if columnIndex >= 0:
-                # Get the header information.  In Java Swing, the
-                # information is not exposed via the description
-                # but is instead a header object, so we fall back
-                # to that if it exists.
-                #
-                # [[[TODO: WDW - the more correct thing to do, I
-                # think, is to look at the column header object.
-                # We've been looking at the description for so
-                # long, though, that we'll give the description
-                # preference for now.]]]
-                #
-                desc = table.getColumnDescription(columnIndex)
-                if not desc:
-                    header = table.getColumnHeader(columnIndex)
-                    if header:
-                        desc = self._script.utilities.displayedText(header)
-                if desc and len(desc):
-                    text = desc
-                    if args['mode'] == 'speech':
-                        if settings.speechVerbosityLevel \
-                           == settings.VERBOSITY_LEVEL_VERBOSE \
-                           and not args.get('formatType', None) \
-                                   in ['basicWhereAmI', 'detailedWhereAmI']:
-                            text = desc + " " + self.getLocalizedRoleName(
-                                obj, pyatspi.ROLE_COLUMN_HEADER)
-                    elif args['mode'] == 'braille':
-                        text = desc + " " + self.getLocalizedRoleName(
-                            obj, pyatspi.ROLE_COLUMN_HEADER)
-                    result.append(text)
+        text = self._script.utilities.displayedText(header)
+        if not text:
+            return result
+
+        roleString =  self.getLocalizedRoleName(obj, pyatspi.ROLE_COLUMN_HEADER)
+        if args.get('mode') == 'speech':
+            if settings.speechVerbosityLevel == settings.VERBOSITY_LEVEL_VERBOSE \
+               and not args.get('formatType') in ['basicWhereAmI', 'detailedWhereAmI']:
+                text = "%s %s" % (text, roleString)
+        elif args.get('mode') == 'braille':
+            text = "%s %s" % (text, roleString)
+
+        result.append(text)
         return result
 
     def _generateTableCell2ChildLabel(self, obj, **args):
@@ -728,19 +684,9 @@ class Generator:
             cellOrder = []
             hasToggle = [False, False]
             for i, child in enumerate(obj):
-                try:
-                    action = child.queryAction()
-                except NotImplementedError:
-                    continue
-                else:
-                    for j in range(0, action.nActions):
-                        # Translators: this is the action name for
-                        # the 'toggle' action. It must be the same
-                        # string used in the *.po file for gail.
-                        #
-                        if action.getName(j) in ["toggle", _("toggle")]:
-                            hasToggle[i] = True
-                            break
+                if self._script.utilities.hasMeaningfulToggleAction(child):
+                    hasToggle[i] = True
+                    break
             if hasToggle[0] and not hasToggle[1]:
                 cellOrder = [ 1, 0 ]
             elif not hasToggle[0] and hasToggle[1]:
@@ -770,20 +716,9 @@ class Generator:
             cellOrder = []
             hasToggle = [False, False]
             for i, child in enumerate(obj):
-                try:
-                    action = child.queryAction()
-                except NotImplementedError:
-                    continue
-                else:
-                    for j in range(0, action.nActions):
-                        # Translators: this is the action name for
-                        # the 'toggle' action. It must be the same
-                        # string used in the *.po file for gail.
-                        #
-                        if action.getName(j) in ["toggle", _("toggle")]:
-                            hasToggle[i] = True
-                            break
-
+                if self._script.utilities.hasMeaningfulToggleAction(child):
+                    hasToggle[i] = True
+                    break
             if hasToggle[0] and not hasToggle[1]:
                 cellOrder = [ 1, 0 ]
             elif not hasToggle[0] and hasToggle[1]:
@@ -806,30 +741,11 @@ class Generator:
             return []
 
         result = []
-        try:
-            parentTable = obj.parent.queryTable()
-        except:
-            return result
-        try:
-            action = obj.queryAction()
-            label = self._script.utilities.displayedText(
-                        self._script.utilities.realActiveDescendant(obj))
-        except NotImplementedError:
-            action = None
-            label = None
-        if action and (label == None or len(label) == 0):
-            index = self._script.utilities.cellIndex(obj)
-            column = parentTable.getColumnAtIndex(index)
-            for j in range(0, action.nActions):
-                # Translators: this is the action name for
-                # the 'toggle' action. It must be the same
-                # string used in the *.po file for gail.
-                #
-                if action.getName(j) in ["toggle",
-                                         _("toggle")]:
-                    accHeader = \
-                        parentTable.getColumnHeader(column)
-                    result.append(accHeader.name)
+        descendant = self._script.utilities.realActiveDescendant(obj)
+        label = self._script.utilities.displayedText(descendant)
+        if not label and self._script.utilities.hasMeaningfulToggleAction(obj):
+            accHeader = self._script.utilities.columnHeaderForCell(obj)
+            result.append(accHeader.name)
         return result
 
     def _generateRealTableCell(self, obj, **args):
@@ -914,7 +830,7 @@ class Generator:
                         cellResult = self._generateRealTableCell(cell, **args)
                         if cellResult and result and self._mode == 'braille':
                             result.append(braille.Region(
-                                    settings.brailleTableCellDelimiter))
+                                object_properties.TABLE_CELL_DELIMITER_BRAILLE))
                         result.extend(cellResult)
             else:
                 result.extend(self._generateRealTableCell(obj, **args))
@@ -928,19 +844,53 @@ class Generator:
     #                                                                   #
     #####################################################################
 
+    def _generateExpandedEOCs(self, obj, **args):
+        """Returns the expanded embedded object characters for an object."""
+        return []
+
+    def _generateSubstring(self, obj, **args):
+        start = args.get('startOffset')
+        end = args.get('endOffset')
+        if start == None or end == None:
+            return []
+
+        substring = self._script.utilities.substring(obj, start, end)
+        if substring and substring.strip() != obj.name \
+           and not self._script.EMBEDDED_OBJECT_CHARACTER in substring:
+            return [substring]
+
+        return []
+
+    def _generateStartOffset(self, obj, **args):
+        return args.get('startOffset')
+
+    def _generateEndOffset(self, obj, **args):
+        return args.get('endOffset')
+
     def _generateCurrentLineText(self, obj, **args ):
         """Returns an array of strings for use by speech and braille
         that represents the current line of text, if
         this is a text object.  [[[WDW - consider returning an empty
         array if this is not a text object.]]]
         """
+        result = self._generateSubstring(obj, **args)
+        if result:
+            return result
+
         [text, caretOffset, startOffset] = self._script.getTextLineAtCaret(obj)
-        return [text]
+        if text and not self._script.EMBEDDED_OBJECT_CHARACTER in text:
+            return [text]
+
+        return []
 
     def _generateDisplayedText(self, obj, **args ):
         """Returns an array of strings for use by speech and braille that
         represents all the text being displayed by the object.
         """
+        result = self._generateSubstring(obj, **args)
+        if result:
+            return result
+
         displayedText = self._script.utilities.displayedText(obj)
         if not displayedText:
             return []
@@ -1056,11 +1006,8 @@ class Generator:
         found.  Otherwise, an empty array is returned.
         """
         result = []
-        text = self._script.utilities.displayedText(
-            self._script.utilities.realActiveDescendant(obj))
-        if text:
-            result = [text]
-        return result
+        rad = self._script.utilities.realActiveDescendant(obj)
+        return self._generateDisplayedText(rad, **args)
 
     def _generateRealActiveDescendantRoleName(self, obj, **args ):
         """Objects, such as tables and trees, can represent individual cells
