@@ -28,7 +28,7 @@ __license__   = "LGPL"
 
 # $ORCA_MAJOR_VERSION.$ORCA_MINOR_VERSION.$ORCA_MICRO_VERSION
 #
-version     = "3.14.3"
+version     = "3.16.3"
 
 # "--prefix" parameter used when configuring the build.
 #
@@ -38,9 +38,9 @@ prefix      = "/home/jd/checkout/orca/bld"
 #
 package     = "orca"
 
-# The name of the data directory (usually "share").
+# The location of the data directory (usually "share").
 #
-datadirname = "share"
+datadir = "${prefix}/share".replace('${prefix}', '/home/jd/checkout/orca/bld')
 
 # The directory where we could find liblouis translation tables.
 #
