@@ -69,7 +69,11 @@ class Utilities(script_utilities.Utilities):
         any text being shown.
         """
 
-        role = obj.getRole()
+        try:
+            role = obj.getRole()
+        except:
+            return ""
+
         if role == pyatspi.ROLE_PUSH_BUTTON and obj.name:
             return obj.name
 
@@ -79,7 +83,10 @@ class Utilities(script_utilities.Utilities):
             if text.strip():
                 return text
 
-        text = script_utilities.Utilities.displayedText(self, obj)
+        try:
+            text = script_utilities.Utilities.displayedText(self, obj)
+        except:
+            return ""
 
         # TODO - JD: This is needed because the default behavior is to fall
         # back on the name, which is bogus. Once that has been fixed, this
@@ -280,11 +287,19 @@ class Utilities(script_utilities.Utilities):
 
     def isSameObject(self, obj1, obj2):
         same = script_utilities.Utilities.isSameObject(self, obj1, obj2)
+        if not same or obj1 == obj2:
+            return same
+
+        # The document frame currently contains just the active page,
+        # resulting in false positives. So for paragraphs, rely upon
+        # the equality check.
+        if obj1.getRole() == obj2.getRole() == pyatspi.ROLE_PARAGRAPH:
+            return False
 
         # Handle the case of false positives in dialog boxes resulting
         # from getIndexInParent() returning a bogus value. bgo#618790.
         #
-        if same and (obj1 != obj2) and not obj1.name \
+        if not obj1.name \
            and obj1.getRole() == pyatspi.ROLE_TABLE_CELL \
            and obj1.getIndexInParent() == obj2.getIndexInParent() == -1:
             top = self.topLevelObject(obj1)
