@@ -27,31 +27,23 @@ __copyright__ = "Copyright (c) 2005-2009 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import importlib
-import logging
-log = logging.getLogger("speech")
-
-import re
 import time
 
-from . import chnames
 from . import debug
+from . import logger
 from . import orca_state
 from . import settings
-from . import sound
 from . import speech_generator
 
 from .acss import ACSS
 
+_logger = logger.getLogger()
+log = _logger.newLog("speech")
+
 # The speech server to use for all speech operations.
 #
 _speechserver = None
 
-# regular expressions for multiCaseStrings
-#
-multiCaseReg1 = re.compile("([a-z]+)([A-Z])")
-multiCaseReg2 = re.compile("([A-Z][A-Z]+)([A-Z][a-z]+)")
-multiCaseReg3 = re.compile("([A-Z])([A-Z][a-z]+)")
-
 def getSpeechServerFactories():
     """Imports all known SpeechServer factory modules.  Returns a list
     of modules that implement the getSpeechServers method, which
@@ -151,13 +143,6 @@ def sayAll(utteranceIterator, progressCallback):
 def _speak(text, acss, interrupt):
     """Speaks the individual string using the given ACSS."""
 
-    if settings.speakMultiCaseStringsAsWords:
-        text = _processMultiCaseString(text)
-    if orca_state.activeScript and orca_state.usePronunciationDictionary:
-        text = orca_state.activeScript.utilities.adjustForPronunciation(text)
-    if settings.speakMultiCaseStringsAsWords:
-        text = _processMultiCaseString(text)
-
     logLine = "SPEECH OUTPUT: '" + text + "'"
     extraDebug = ""
     if acss in list(settings.voices.values()):
@@ -186,7 +171,7 @@ def speak(content, acss=None, interrupt=True):
     if settings.silenceSpeech:
         return
 
-    validTypes = (str, list, sound.Sound, speech_generator.Pause,
+    validTypes = (str, list, speech_generator.Pause,
                   speech_generator.LineBreak, ACSS)
     error = "bad content sent to speech.speak: '%s'"
     if not isinstance(content, validTypes):
@@ -194,16 +179,8 @@ def speak(content, acss=None, interrupt=True):
         debug.println(debug.LEVEL_WARNING, error % content)
         return
 
-    # We will not interrupt a key echo in progress.
-    #
-    if orca_state.lastKeyEchoTime:
-        interrupt = interrupt \
-            and ((time.time() - orca_state.lastKeyEchoTime) > 0.5)
-
     if isinstance(content, str):
         _speak(content, acss, interrupt)
-    elif isinstance(content, sound.Sound):
-        content.play()
     if not isinstance(content, list):
         return
 
@@ -237,9 +214,6 @@ def speak(content, acss=None, interrupt=True):
             activeVoice = newVoice
             toSpeak = newItemsToSpeak
 
-        if isinstance(element, sound.Sound):
-            element.play()
-
     if toSpeak:
         string = " ".join(toSpeak)
         _speak(string, activeVoice, interrupt)
@@ -250,14 +224,10 @@ def speakKeyEvent(event):
     Arguments:
     - event: input_event.KeyboardEvent to speak.
     """
+
     if settings.silenceSpeech:
         return
 
-    if _speechserver:
-        _speechserver.speakKeyEvent(event)
-        return
-
-    # This should only happen during regression tests.
     keyname = event.getKeyName()
     lockingStateString = event.getLockingStateString()
     msg = "%s %s" % (keyname, lockingStateString)
@@ -265,6 +235,9 @@ def speakKeyEvent(event):
     debug.println(debug.LEVEL_INFO, logLine)
     log.info(logLine)
 
+    if _speechserver:
+        _speechserver.speakKeyEvent(event)
+
 def speakCharacter(character, acss=None):
     """Speaks a single character immediately.
 
@@ -279,9 +252,8 @@ def speakCharacter(character, acss=None):
     if settings.silenceSpeech:
         return
 
-    spokenCharacter = chnames.getCharacterName(character)
-    debug.println(debug.LEVEL_INFO, "SPEECH OUTPUT: '" + spokenCharacter + "'")
-    log.info("SPEECH OUTPUT: '%s'" % spokenCharacter)
+    debug.println(debug.LEVEL_INFO, "SPEECH OUTPUT: '" + character + "'")
+    log.info("SPEECH OUTPUT: '%s'" % character)
 
     if _speechserver:
         _speechserver.speakCharacter(character, acss=acss)
@@ -307,10 +279,6 @@ def stop():
 def updateCapitalizationStyle(script=None, inputEvent=None):
     if _speechserver:
         _speechserver.updateCapitalizationStyle()
-    else:
-        logLine = "SPEECH OUTPUT: 'capitalization style' updated"
-        debug.println(debug.LEVEL_INFO, logLine)
-        log.info(logLine)
 
     return True
 
@@ -319,20 +287,12 @@ def updatePunctuationLevel(script=None, inputEvent=None):
 
     if _speechserver:
         _speechserver.updatePunctuationLevel()
-    else:
-        logLine = "SPEECH OUTPUT: 'punctuation level' updated"
-        debug.println(debug.LEVEL_INFO, logLine)
-        log.info(logLine)
 
     return True
 
 def increaseSpeechRate(script=None, inputEvent=None):
     if _speechserver:
         _speechserver.increaseSpeechRate()
-    else:
-        logLine = "SPEECH OUTPUT: 'faster'"
-        debug.println(debug.LEVEL_INFO, logLine)
-        log.info(logLine)
 
     return True
 
@@ -349,20 +309,12 @@ def decreaseSpeechRate(script=None, inputEvent=None):
 def increaseSpeechPitch(script=None, inputEvent=None):
     if _speechserver:
         _speechserver.increaseSpeechPitch()
-    else:
-        logLine = "SPEECH OUTPUT: 'higher'"
-        debug.println(debug.LEVEL_INFO, logLine)
-        log.info(logLine)
 
     return True
 
 def decreaseSpeechPitch(script=None, inputEvent=None):
     if _speechserver:
         _speechserver.decreaseSpeechPitch()
-    else:
-        logLine = "SPEECH OUTPUT: 'lower'"
-        debug.println(debug.LEVEL_INFO, logLine)
-        log.info(logLine)
 
     return True
 
@@ -404,13 +356,3 @@ def test():
                 server.shutdown()
             except:
                 debug.printException(debug.LEVEL_OFF)
-
-def _processMultiCaseString(string):
-    """Helper function, applies the regexes to split multiCaseStrings
-    to multiple words.
-    """
-
-    string = multiCaseReg1.sub('\\1 \\2', string)
-    string = multiCaseReg2.sub('\\1 \\2', string)
-    string = multiCaseReg3.sub('\\1 \\2', string)    
-    return string
